clear functions

Tmax=.0599;

net=readNet('dimerization.net');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% first order model
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

disp('first-order model')
[mdyn1,x10]=closureDynamics(net,1,'derMatch','fun');
options=odeset('RelTol',1e-6,'AbsTol',1e-6,'Stats','on');
[t1,mu1]=ode23s(@(t,x)fun(x),[0,Tmax],x10,options);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% second order model
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

disp('second-order model')
[mdyn2,x20]=closureDynamics(net,2,'derMatch','fun');
[t2,mu2]=ode23s(@(t,x)fun(x),[0,Tmax],x20,options);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% third order model
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

disp('third-order model')
[mdyn3,x30]=closureDynamics(net,3,'derMatch','fun');
[t3,mu3]=ode23s(@(t,x)fun(x),[0,Tmax],x30,options);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% ODE plots
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

figure(1);clf
subplot(1,3,1)
h1=plotCMoments(net,mdyn1,t1,mu1,'X1','k-');
hold on
h2=plotCMoments(net,mdyn2,t2,mu2,'X1','b-');
h3=plotCMoments(net,mdyn3,t3,mu3,'X1','r-');
hold off
grid on
legend([h1(1),h2(1),h3(1)],'1st order','2nd order','3rd order','location','best');
[a1,s1]=getCMoments(net,mdyn3,mu3(end,:),'X1');
[a2,s2]=getCMoments(net,mdyn3,mu3(end,:),'X2');
title(sprintf('Moment truncation: X1\\pm{}Std[X1]\nE[X1]=%0.2g, Std[X1]=%0.2g (3rd order)\nE[X2]=%0.2g, Std[X2]=%0.2g (3rd order)',a1,s1,a2,s2));
drawnow

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% SSA
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
net2stochKit(net,'ProblemDefinition',x10);
%type ProblemDefinition.cpp

cmd='make single stats';
fprintf('executing: ''%s''...\n',cmd);
rc=system(cmd);
if rc
  fprintf('\nFailed to make stochKit executable. Will not run Monte Simulations\n');
  return
end

cmd=sprintf('rm -f single.txt;nice single %0.2f single.txt',Tmax);
fprintf('executing: ''%s''...\n',cmd);
rc=system(cmd);
tx=load('single.txt');

subplot(1,3,2)
plot(tx(:,1),tx(:,2:end))
legend('X1','X2','location','best')
title('Monte Carlo sample')
grid on
drawnow

cmd=sprintf('rm -f stats.txt;nice stats %0.2f 50000 stats.txt',Tmax);
fprintf('executing: ''%s''...\n',cmd);
rc=system(cmd);
xfinal=load('stats.txt');

subplot(1,3,3)
[n,x]=hist(xfinal(:,1),20);
bar(x,n);
%set(gca,'yscale','log')
xx=min(x):(max(x)-min(x))/1000:max(x);
pdf=getDistribution(net,mdyn3,mu3(end,:),'X1',xx,'lognormal');
hold on
plot(xx,size(xfinal,1)*pdf*(x(2)-x(1)))
hold off
title(sprintf('Monte Carlo: X1\nE[X1]=%0.2g, Std[X1]=%0.2g\nE[X2]=%0.2g, Std[X2]=%0.2g',mean(xfinal(:,1)),std(xfinal(:,1)),mean(xfinal(:,2)),std(xfinal(:,2))))

